﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Assets.Scripts.Common;
using UnityEngine;

namespace Assets.Scripts.CircleComponents
{
    class Propeller : MonoBehaviour
    {
        public float Force = 5;
        public float FuelConsumption = 0.1f;

        public ParticleSystem ParticleSystem;
        public FuelTank FuelTank;

        public bool IsOn { get; private set; }

        private Rigidbody2D _rigidbody2D;

        public void Awake()
        {
            _rigidbody2D = GetComponentInParent<Rigidbody2D>();
            ParticleSystem.gameObject.SetActive(false);
        }

        public void FixedUpdate()
        {
            if (IsOn)
            {
                float requiredFuel = FuelConsumption * Time.deltaTime;

                if (FuelTank.AvailableFuel >= requiredFuel)
                {
                    FuelTank.AvailableFuel -= requiredFuel;

                    _rigidbody2D.AddForce(
                        Helper.GetDirectionVector(transform.rotation.eulerAngles.z * Mathf.Deg2Rad) * Force);
                }
            }
        }

        public void Fire()
        {
            IsOn = true;
            ParticleSystem.gameObject.SetActive(true);
        }

        public void Stop()
        {
            IsOn = false;
            ParticleSystem.gameObject.SetActive(false);
        }
    }
}
