﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Assets.Scripts.Common
{
    abstract class AnimationBase : MonoBehaviour
    {
        public int currentFrame
        {
            get { return _currentFrame; }
            set
            {
                if (_currentFrame != value)
                {
                    _currentFrame = value;
                    RefreshFrame();
                }
            }
        }
        public List<Sprite> frames;
        public float sampleRate = 12;
        public bool isLooped;
        public bool isPaused;

        private double _timer;
        private int _totalFrameCount;
        private int _startFrame;
        private int _endFrame;
        private int _loopCounter;
        private bool _finishedLoop;
        private bool _isInitialized;
        private int _currentFrame;
        private int _activeFrameRange;

        public void Start()
        {
            Initialize();
        }

        public void Update()
        {
            if (!_isInitialized && frames.Count > 0)
            {
                Initialize();
                if (!_isInitialized)
                {
                    return;
                }
            }

            if (!isPaused || (isLooped && _finishedLoop))
            {
                //                CurrentFrame = (int)(_timer * sampleRate % _frameRange);

                int currentLoop = (int)(_timer * sampleRate / _activeFrameRange);

                if (!isLooped && currentLoop > _loopCounter)
                {
                    _finishedLoop = true;
                    currentFrame = _endFrame;
                }
                else
                {
                    currentFrame = (int)(_timer * sampleRate % _activeFrameRange);
                }
                _loopCounter = currentLoop;
                _timer += Time.deltaTime;
            }
        }

        public void SetFrameRange(int startFrame, int endFrame)
        {
            _startFrame = startFrame;
            _endFrame = endFrame;
            _activeFrameRange = (_endFrame - _startFrame) + 1;
            _timer = 0;
            currentFrame = _startFrame;
        }

        protected abstract void RefreshFrame();

        private void Initialize()
        {
            if (frames != null && frames.Count > 0)
            {
                _totalFrameCount = _activeFrameRange = frames.Count;
                _endFrame = _totalFrameCount - 1;
                RefreshFrame();
                _isInitialized = true;
            }
        }
    }
}