﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Assets.Scripts.Common
{
    [RequireComponent(typeof(AudioSource))]
    public class Click : MonoBehaviour
    {

        public static Click Instance
        {
            get
            {
                if (_instance == null)
                {
                    _instance = FindObjectOfType<Click>();

                    //Tell unity not to destroy this object when loading a new scene!
                    DontDestroyOnLoad(_instance.gameObject);
                }

                return _instance;
            }
        }

        private static Click _instance;
        private AudioSource _audioSource;

        public void Awake()
        {
            if (_instance == null)
            {
                //If I am the first instance, make me the Singleton
                _instance = this;
                DontDestroyOnLoad(this);
            }
            else
            {
                //If a Singleton already exists and you find
                //another reference in scene, destroy it!
                if (this != _instance)
                    Destroy(gameObject);
            }
        }

        public void Play()
        {
            if (_audioSource == null)
            {
                _audioSource = GetComponent<AudioSource>();
            }
            _audioSource.Play();
        }
    }
}