﻿using System;
using System.Globalization;
using UnityEngine;

namespace Assets.Scripts.Common
{
    public struct Int2 : IEquatable<Int2>
    {
        public int x { get; set; }
        public int y { get; set; }

        public Int2(int value)
            : this()
        {
            x = value;
            y = value;
        }

        public Int2(int x, int y)
            : this()
        {
            this.x = x;
            this.y = y;
        }

        public double Magnitude()
        {
            return Math.Sqrt(Math.Pow(x, 2) + Math.Pow(y, 2));
        }

        public static Int2 Zero
        {
            get { return new Int2(0, 0); }
        }

        public static Int2 One
        {
            get { return new Int2(1, 1); }
        }

        public static explicit operator Int2(Vector2 vector)
        {
            return new Int2((int)vector.x, (int)vector.y);
        }

        public bool Equals(Int2 other)
        {
            return (x == other.x) && (y == other.y);
        }
        public override bool Equals(object other)
        {
            if (!(other is Int2))
            {
                return false;
            }

            return Equals((Int2)other);
        }

        public static Int2 operator +(Int2 value1, Int2 value2)
        {
            return new Int2(value1.x + value2.x, value1.y + value2.y);
        }
        public static Int2 operator -(Int2 value1, Int2 value2)
        {
            return new Int2(value1.x - value2.x, value1.y - value2.y);
        }

        public static Int2 operator *(Int2 value1, int value2)
        {
            return new Int2(value1.x * value2, value1.y * value2);
        }
        public static Int2 operator *(int value1, Int2 value2)
        {
            return value2 * value1;
        }

        public static Int2 operator /(Int2 value1, int value2)
        {
            return new Int2(value1.x / value2, value1.y / value2);
        }
        public static Int2 operator /(int value1, Int2 value2)
        {
            return value2 / value1;
        }

        public static Int2 operator ++(Int2 value)
        {
            return new Int2(value.x + 1, value.y + 1);
        }
        public static Int2 operator --(Int2 value)
        {
            return new Int2(value.x - 1, value.y - 1);
        }

        public static bool operator ==(Int2 value1, Int2 value2)
        {
            return value1.x == value2.x && value1.y == value2.y;
        }
        public static bool operator !=(Int2 value1, Int2 value2)
        {
            return !(value1 == value2);
        }

        public override int GetHashCode()
        {
            return x.GetHashCode() ^ y.GetHashCode();
        }

        public static implicit operator Vector2(Int2 value)
        {
            return new Vector2(value.x, value.y);
        }

        public override string ToString()
        {
            CultureInfo currentCulture = CultureInfo.CurrentCulture;

            return string.Format(
              currentCulture, "{{X:{0} Y:{1}}}",
              x.ToString(currentCulture),
              y.ToString(currentCulture)
            );
        }
    }
}
