﻿using System;
using Assets.Scripts.Common;
using UnityEngine;
using UnityEngine.UI;

namespace Assets.Scripts.Common
{
    public enum MessageBoxButtons
    {
        OK, OKCancel, YesNo
    }

    public enum MessageBoxButtonOrder
    {
        NegativePositive, PositiveNegative
    }

    public enum DialogResult
    {
        Positive, Negative
    }

    public delegate void MessageBoxCallBack(DialogResult dialogResult);

    public class MessageBox : MonoBehaviour
    {
        public Text Text;
        public Button LeftButton;
        public Button RightButton;

        private MessageBoxCallBack _callBack;

        public void OnEnable()
        {
            BackButtonReceiver.SetActiveReceiver(gameObject);
        }

        public void OnDisable()
        {
            BackButtonReceiver.ReleaseReceiver(gameObject);
        }

        public void Initialize(string msg, MessageBoxButtons buttons,
            MessageBoxCallBack callBack, MessageBoxButtonOrder messageBoxButtonOrder)
        {
            _callBack = callBack;
            Text.text = msg;

            Button positiveButton;
            Button negativeButton;
            switch (messageBoxButtonOrder)
            {
                case MessageBoxButtonOrder.PositiveNegative:
                    positiveButton = LeftButton;
                    negativeButton = RightButton;
                    break;
                case MessageBoxButtonOrder.NegativePositive:
                    positiveButton = RightButton;
                    negativeButton = LeftButton;
                    break;
                default:
                    throw new ArgumentOutOfRangeException("messageBoxButtonOrder");
            }

            switch (buttons)
            {
                case MessageBoxButtons.OK:
                    negativeButton.gameObject.SetActive(false);
                    positiveButton.GetComponentInChildren<Text>().text = "Ok";
                    break;
                case MessageBoxButtons.OKCancel:
                    negativeButton.GetComponentInChildren<Text>().text = "Cancel";
                    positiveButton.GetComponentInChildren<Text>().text = "Ok";
                    break;
                case MessageBoxButtons.YesNo:
                    negativeButton.GetComponentInChildren<Text>().text = "No";
                    positiveButton.GetComponentInChildren<Text>().text = "Yes";
                    break;
                default:
                    throw new ArgumentOutOfRangeException("buttons");
            }

            negativeButton.onClick.AddListener(() =>
            {
                Click.Instance.Play();
                if (callBack != null)
                {
                    callBack(DialogResult.Negative);
                }
                Destroy(gameObject);
            });
            positiveButton.onClick.AddListener(() =>
            {
                Click.Instance.Play();
                if (callBack != null)
                {
                    callBack(DialogResult.Positive);
                }
                Destroy(gameObject);
            });
        }

        public void Update()
        {
            if (Input.GetKeyUp(KeyCode.Escape) && BackButtonReceiver.ActiveReceiver == gameObject)
            {
                Click.Instance.Play();
                if (_callBack != null)
                {
                    _callBack(DialogResult.Negative);
                }
                Destroy(gameObject);
            }
        }
    }
}
