﻿using UnityEngine;
using UnityEngine.EventSystems;

namespace Assets.Scripts.Common
{
    class MessageBoxLauncher : MonoBehaviour
    {
        public static MessageBoxLauncher Instance
        {
            get
            {
                if (_instance == null)
                {
                    _instance = FindObjectOfType<MessageBoxLauncher>();

                    //Tell unity not to destroy this object when loading a new scene!
                    DontDestroyOnLoad(_instance.gameObject);
                }

                return _instance;
            }
        }

        public MessageBox MessageBoxPrefab;

        private static MessageBoxLauncher _instance;

        public void Awake()
        {
            if (_instance == null)
            {
                //If I am the first instance, make me the Singleton
                _instance = this;
                DontDestroyOnLoad(this);
            }
            else
            {
                //If a Singleton already exists and you find
                //another reference in scene, destroy it!
                if (this != _instance)
                    Destroy(gameObject);
            }
        }

        public void Show(string msg, MessageBoxButtons buttons,
            MessageBoxCallBack callBack = null,
            MessageBoxButtonOrder messageBoxButtonOrder = MessageBoxButtonOrder.NegativePositive)
        {
            EventSystem eventSystem = FindObjectOfType<EventSystem>();
            if (eventSystem == null)
            {
                GameObject eventsystem = new GameObject();
                eventSystem.name = "EventSystem";
                eventsystem.AddComponent<EventSystem>();
                eventsystem.AddComponent<StandaloneInputModule>();
                eventsystem.AddComponent<TouchInputModule>();
            }

            var msgBox = Instantiate(MessageBoxPrefab);
            msgBox.Initialize(msg, buttons, callBack, messageBoxButtonOrder);
        }
    }
}
