﻿using System;

namespace Assets.Scripts.Common
{
    public struct Speed
    {
        public static Speed Zero
        {
            get { return zero; }
        }

        public float Distance
        {
            get { return _distance; }
            private set
            {
                _distance = value;
            }
        }

        public float Duration
        {
            get { return _duration; }
            private set
            {
                _duration = value;
            }
        }

        private static readonly Speed zero = new Speed();
        private float _distance;
        private float _duration;

        public Speed(float distance, float duration)
            : this()
        {
            if (distance < 0)
            {
                throw new Exception("distance must be >=0");
            }

            SetSpeed(distance, duration);
        }

        public void SetSpeed(float distance, float duration)
        {
            if (distance < 0)
            {
                throw new Exception("distance must be >=0");
            }

            Duration = duration;
            Distance = distance;
        }

        public float GetDistancePerTime(float time)
        {
            if (Math.Abs(Duration) < 0.0001f)
            {
                return 0;
            }

            return Distance * (time / Duration);
        }
        public float GetTimeToTravel(float distance)
        {
            return Math.Abs(distance) / Distance * Duration;
        }
    }
}
