﻿using System;

namespace Assets.Scripts.Common
{
    public class StateChangedEventArgs : EventArgs
    {
        public readonly IState PreviousState;
        public readonly IState CurrentState;

        public StateChangedEventArgs(IState currentState, IState previousState)
        {
            CurrentState = currentState;
            PreviousState = previousState;
        }
    }

    public class StateManager<T> where T : class, IState
    {
        public T CurrentState
        {
            get { return _currentState; }
        }

        public event EventHandler<StateChangedEventArgs> StateChanged;

        private T _currentState;

        public bool SetState(T state, object obj)
        {
//            if (state == _currentState) return false;

            IState prevState = _currentState;

            if (_currentState != null)
            {
                _currentState.Deactivate();
            }

            _currentState = state;
            if (_currentState != null)
            {
                _currentState.Activate(obj);
            }

            if (StateChanged != null)
            {
                StateChanged(this, new StateChangedEventArgs(_currentState, prevState));
            }

            return true;
        }

        public bool SetState(T state)
        {
            return SetState(state, null);
        }
    }
}
