﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Assets.Scripts.Data
{
    public static class GlobalSettings
    {
        public static bool SFXEnabled
        {
            get { return IsPropertyActive(SFXPropertyName); }
            set
            {
                PlayerPrefs.SetInt(SFXPropertyName, value ? 1 : 0);
                PlayerPrefs.Save();
                OnPropertyChanged(SFXPropertyName);
            }
        }
        public static bool MusicEnabled
        {
            get { return IsPropertyActive(MusicPropertyName); }
            set
            {
                PlayerPrefs.SetInt(MusicPropertyName, value ? 1 : 0);
                PlayerPrefs.Save();
                OnPropertyChanged(MusicPropertyName);
            }
        }
        public static bool Rated
        {
            get { return IsPropertyActive(ReviewPropertyName); }
            set
            {
                PlayerPrefs.SetInt(ReviewPropertyName, value ? 1 : 0);
                PlayerPrefs.Save();
                OnPropertyChanged(ReviewPropertyName);
            }
        }
        public static bool ViewedTutorial
        {
            get { return IsPropertyActive(ViewedTutorialPropertyName); }
            set
            {
                PlayerPrefs.SetInt(ViewedTutorialPropertyName, value ? 1 : 0);
                PlayerPrefs.Save();
                OnPropertyChanged(ViewedTutorialPropertyName);
            }
        }

        public static bool FacebookLiked
        {
            get { return IsPropertyActive(FacebookLikePropertyName); }
            set
            {
                PlayerPrefs.SetInt(FacebookLikePropertyName, value ? 1 : 0);
                PlayerPrefs.Save();
                OnPropertyChanged(FacebookLikePropertyName);
            }
        }

        public static bool IsFirstLaunch { get; private set; }

        public static readonly string SFXPropertyName = "SoundEnabled";
        public static readonly string MusicPropertyName = "MusicEnabled";
        public static readonly string ReviewPropertyName = "Rated";
        public static readonly string FacebookLikePropertyName = "FacebookLiked";
        public static readonly string IsFirstLaunchPropertyName = "FirstLaunch";
        public static readonly string ViewedTutorialPropertyName = "ViewedTutorial";

        public static event PropertyChangedEventHandler PropertyChanged;

        static GlobalSettings()
        {
            if (!PlayerPrefs.HasKey(SFXPropertyName))
            {
                PlayerPrefs.SetInt(SFXPropertyName, 1);
            }
            if (!PlayerPrefs.HasKey(MusicPropertyName))
            {
                PlayerPrefs.SetInt(MusicPropertyName, 1);
            }

            if (!PlayerPrefs.HasKey(IsFirstLaunchPropertyName))
            {
                PlayerPrefs.SetInt(IsFirstLaunchPropertyName, 0);
                IsFirstLaunch = true;
            }
            else
            {
                IsFirstLaunch = false;
            }

            PlayerPrefs.Save();
        }

        private static void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null) handler(null, new PropertyChangedEventArgs(propertyName));
        }

        private static bool IsPropertyActive(string propertyName)
        {
            if (PlayerPrefs.HasKey(propertyName))
            {
                return PlayerPrefs.GetInt(propertyName) == 1;
            }
            return false;
        }
    }
}
