﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CodeStage.AntiCheat.ObscuredTypes;
using Newtonsoft.Json;

namespace Assets.Scripts.Data
{
    class PlayerManager
    {
        public static PlayerManager Instance
        {
            get { return _instance ?? (_instance = new PlayerManager()); }
        }

        public event EventHandler PlayerDataChanged;

        public PlayerData CachedPlayerDataCopy;

        private static PlayerManager _instance;
        private readonly object _saveLock = new object();

        private PlayerManager()
        {
            var playerData = GetPlayerDataFromDisk();
            if (playerData == null)
            {
                playerData = new PlayerData();
                playerData.OnLoad();
                SavePlayerData(playerData);
            }

            CachedPlayerDataCopy = playerData;
        }

        public PlayerData GetPlayerDataFromDisk()
        {
            if (!ObscuredPrefs.HasKey("PlayerData"))
            {
                return null;
            }

            PlayerData playerData;

            try
            {
                playerData = JsonConvert.DeserializeObject<PlayerData>(ObscuredPrefs.GetString("PlayerData"));
            }
            catch (Exception)
            {
                playerData = null;
            }

            if (playerData != null)
            {
                playerData.OnLoad();
            }

            return playerData;
        }

        public void DebugReset()
        {
            var playerData = new PlayerData();
            playerData.OnLoad();
            SavePlayerData(playerData);
        }

        public void DebugUnlockAllLevels()
        {
            PlayerData playerData = GetPlayerDataFromDisk();
            foreach (var level in CachedLevelsData.Instance.LevelsData.LevelSequence)
            {
                if (!playerData.CompletedLevels.Contains(level))
                {
                    playerData.CompletedLevels.Add(level);
                }
            }
            SavePlayerData(playerData);
        }

        public void SetLevelComplete(Guid levelID)
        {
            PlayerData playerData = GetPlayerDataFromDisk();
            if (!playerData.CompletedLevels.Contains(levelID))
            {
                playerData.CompletedLevels.Add(levelID);
            }

            SavePlayerData(playerData);
        }

        private void SavePlayerData(PlayerData playerData)
        {
            if (playerData == null)
            {
                return;
            }

            lock (_saveLock)
            {
                playerData.Version = GlobalConstants.BuildVersion;
                playerData.OnSave();
                ObscuredPrefs.SetString("PlayerData", JsonConvert.SerializeObject(playerData));
                ObscuredPrefs.Save();

                CachedPlayerDataCopy = playerData;

                if (PlayerDataChanged != null)
                {
                    PlayerDataChanged(this, EventArgs.Empty);
                }
            }
        }
    }
}
