﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DG.Tweening;
using UnityEngine;
using UnityEngine.UI;

namespace Assets.Scripts.FuelCellComponents
{
    class FuelCellsSequencer : MonoBehaviour
    {
        public int TotalFuelCells { get; private set; }
        public int ConsumedFuelCells { get; private set; }
        public FuelCell ActiveFuelCell { get; private set; }

        private List<FuelCell> _fuelCells;

        public void Awake()
        {
            _fuelCells = GetComponentsInChildren<FuelCell>()
                .OrderBy(cell => cell.GetComponent<FuelCellSequenceOrder>().Order).ToList();

            for (int i = 1; i < _fuelCells.Count; i++)
            {
                if (_fuelCells[i].GetComponentInChildren<FuelCellSequenceOrder>()
                    == _fuelCells[i - 1].GetComponentInChildren<FuelCellSequenceOrder>())
                {
                    throw new Exception();
                }
            }

            for (int i = 0; i < _fuelCells.Count; i++)
            {
                _fuelCells[i].OnConsumed += FuelCellOnConsumed;
                _fuelCells[i].GetComponentInChildren<Text>().text = (_fuelCells.Count - i).ToString();
            }

            TotalFuelCells = _fuelCells.Count;
            foreach (var fuelCell in _fuelCells)
            {
                fuelCell.gameObject.SetActive(false);
            }
        }

        private void FuelCellOnConsumed(object sender, EventArgs eventArgs)
        {
            ConsumedFuelCells++;
            FuelCell fuelCell = (FuelCell)sender;
            fuelCell.OnConsumed -= FuelCellOnConsumed;
        }

        public void Start()
        {
            StartCoroutine(Refresh());
        }

        private void PresentFuelCell(FuelCell fuelCell)
        {
            Vector3 originalScale = fuelCell.transform.localScale;
            fuelCell.transform.localScale *= 1.2f;
            fuelCell.transform.DOScale(originalScale, 0.3f).SetEase(Ease.OutQuad);
        }

        private IEnumerator Refresh()
        {
            while (true)
            {
                _fuelCells.RemoveAll(cell => cell == null);
                if (ActiveFuelCell == null)
                {
                    if (_fuelCells.Count > 0)
                    {
                        ActiveFuelCell = _fuelCells[0];
                        ActiveFuelCell.gameObject.SetActive(true);
                        _fuelCells.RemoveAt(0);
                        PresentFuelCell(ActiveFuelCell);
                    }
                    else
                    {
                        yield break;
                    }
                }
                else
                {
                    yield return new WaitForSeconds(0.05f);
                }
            }
        }
    }
}
