﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Assets.Scripts.Data;
using UnityEngine;
using UnityEngine.SceneManagement;

namespace Assets.Scripts.Game
{
    public enum Scene
    {
        Init, Main, Arcade, Tutorial
    }

    static class GameHelper
    {
        public static void Share(byte[] pic, string textToPost)
        {
#if UNITY_ANDROID
            byte[] dataToSave = pic;
            string destination = Path.Combine(
                Application.persistentDataPath, DateTime.Now.ToString("PopcornChefScreenshot") + ".png");
            File.WriteAllBytes(destination, dataToSave);

            AndroidJavaClass intentClass = new AndroidJavaClass("android.content.Intent");
            AndroidJavaObject intentObject = new AndroidJavaObject("android.content.Intent");
            intentObject.Call<AndroidJavaObject>("setAction", intentClass.GetStatic<string>("ACTION_SEND"));
            AndroidJavaClass uriClass = new AndroidJavaClass("android.net.Uri");
            AndroidJavaObject uriObject = uriClass.CallStatic<AndroidJavaObject>("parse", "file://" + destination);
            intentObject.Call<AndroidJavaObject>("putExtra", intentClass.GetStatic<string>("EXTRA_STREAM"), uriObject);
            intentObject.Call<AndroidJavaObject>("putExtra", intentClass.GetStatic<string>("EXTRA_TEXT"), textToPost);

            intentObject.Call<AndroidJavaObject>("setType", "image/jpeg");
            AndroidJavaClass unity = new AndroidJavaClass("com.unity3d.player.UnityPlayer");
            AndroidJavaObject currentActivity = unity.GetStatic<AndroidJavaObject>("currentActivity");

            // option one WITHOUT chooser:
            AndroidJavaObject jChooser = intentClass.CallStatic<AndroidJavaObject>(
                "createChooser", intentObject, "Share Via");
            currentActivity.Call("startActivity", jChooser);
#endif
        }

        public static void LaunchFacebookPage()
        {
            if (Application.isWebPlayer)
            {
                Application.ExternalEval("window.open('https://www.facebook.com/heat9games','_blank')");
            }
            else
            {
                Application.OpenURL("https://www.facebook.com/heat9games");
            }
        }

        public static void LaunchAppRate()
        {
            Application.OpenURL("market://details?id=com.ramsayamarin.popcornchef");
        }

        public static Scene GetSceneNameFromIndex(int index)
        {
            switch (index)
            {
                case 0:
                    return Scene.Init;
                case 1:
                    return Scene.Main;
                case 2:
                    return Scene.Arcade;
                case 3:
                    return Scene.Tutorial;
                default:
                    throw new Exception();
            }
        }

        public static int GetSceneIndexFromName(Scene scene)
        {
            switch (scene)
            {
                case Scene.Init:
                    return 0;
                case Scene.Main:
                    return 1;
                case Scene.Arcade:
                    return 2;
                case Scene.Tutorial:
                    return 3;
                default:
                    throw new ArgumentOutOfRangeException("scene");
            }
        }

        public static Dictionary<Guid, LevelState> GetAllLevelsState()
        {
            PlayerData playerData = PlayerManager.Instance.GetPlayerDataFromDisk();

            Dictionary<Guid, LevelState> levelsStates =
                new Dictionary<Guid, LevelState>(CachedLevelsData.Instance.LevelsData.LevelSequence.Count);

            for (int i = 0; i < CachedLevelsData.Instance.LevelsData.LevelSequence.Count; i++)
            {
                if (i == 0)
                {
                    if (playerData.CompletedLevels.Contains(
                        CachedLevelsData.Instance.LevelsData.LevelSequence[i]))
                    {
                        levelsStates.Add(CachedLevelsData.Instance.LevelsData.LevelSequence[i], LevelState.Complete);
                    }
                    else
                    {
                        levelsStates.Add(CachedLevelsData.Instance.LevelsData.LevelSequence[i], LevelState.Incomplete);
                    }
                }
                else
                {
                    if (playerData.CompletedLevels.Contains(CachedLevelsData.Instance.LevelsData.LevelSequence[i]))
                    {
                        levelsStates.Add(CachedLevelsData.Instance.LevelsData.LevelSequence[i], LevelState.Complete);
                    }
                    else if (playerData.CompletedLevels.Contains(CachedLevelsData.Instance.LevelsData.LevelSequence[i - 1]))
                    {
                        levelsStates.Add(CachedLevelsData.Instance.LevelsData.LevelSequence[i], LevelState.Incomplete);
                    }
                    else
                    {
                        levelsStates.Add(CachedLevelsData.Instance.LevelsData.LevelSequence[i], LevelState.Locked);
                    }
                }
            }

            return levelsStates;
        }
    }
}
