﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using Assets.Scripts.CircleComponents;
using Assets.Scripts.Common;
using Assets.Scripts.Data;
using Assets.Scripts.FuelCellComponents;
using Assets.Scripts.OptionsScreenUtils;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

namespace Assets.Scripts.Game
{
    partial class GameManager : MonoBehaviour, IGameManager
    {
        private interface IGameManagerState : IState
        {
            void Update();
        }

        public Guid LevelID { get; private set; }

        public GameManagerState State
        {
            get
            {
                if (_stateManager.CurrentState == _stateGameOver)
                {
                    return GameManagerState.GameOver;
                }
                if (_stateManager.CurrentState == _statePlay)
                {
                    return GameManagerState.Playing;
                }
                return GameManagerState.Idle;
            }
        }

        public event EventHandler StateChanged;
        public event EventHandler<GameOverEventArgs> OnGameOver;

        private PressDetector _pressDetector;
        private Circle _circle;
        private FuelCellsSequencer _fuelCellsSequencer;
        private bool _gameOverFired;

        private readonly StateManager<IGameManagerState> _stateManager = new StateManager<IGameManagerState>();
        private StateAwaitingStart _stateAwaitingStart;
        private StatePlay _statePlay;
        private StateGameOver _stateGameOver;

        public void Awake()
        {
            LevelID = new Guid(SceneManager.GetActiveScene().name);
            ActiveLevelManager.Instance.SetActiveLevel(LevelID);

            _pressDetector = FindObjectOfType<PressDetector>();
            _fuelCellsSequencer = FindObjectOfType<FuelCellsSequencer>();

            _stateAwaitingStart = new StateAwaitingStart(this);
            _statePlay = new StatePlay(this);
            _stateGameOver = new StateGameOver(this);

            _stateManager.StateChanged += (sender, args) =>
            {
                if (StateChanged != null)
                {
                    StateChanged(this, EventArgs.Empty);
                }
            };
        }

        public void Start()
        {
            _circle = FindObjectOfType<Circle>();
            _stateManager.SetState(_stateAwaitingStart);
        }

        public void Update()
        {
            _stateManager.CurrentState.Update();
        }

        private void EndGame(bool isSuccessful)
        {
            if (!_gameOverFired)
            {
                if (isSuccessful)
                {
                    PlayerManager.Instance.SetLevelComplete(LevelID);
                    ActiveLevelManager.Instance.SetLevelCompleted(LevelID);
                }

                _gameOverFired = true;
                if (OnGameOver != null)
                {
                    OnGameOver(this, new GameOverEventArgs(isSuccessful));
                }
            }
        }
    }
}
