﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Assets.Scripts.Common;
using Assets.Scripts.Data;
using Assets.Scripts.OptionsScreenUtils;
using DG.Tweening;
using UnityEngine;
using UnityEngine.UI;

namespace Assets.Scripts.Game
{
    [RequireComponent(typeof(UIToggler))]
    class GameNavBar : MonoBehaviour
    {
        public ScreenManager ScreenManager;
        public OptionsScreen OptionsScreen;
        public Text LevelIndexText;
        public Button OptionsButton;

        private IGameManager _gameManager;
        private UIToggler _uiToggler;
        
        public void Awake()
        {
            _uiToggler = GetComponent<UIToggler>();
        }

        public void Start()
        {
            GameObject gameManagerGameObj = GameObject.FindGameObjectWithTag(GlobalConstants.GameManagerTag);
            _gameManager = gameManagerGameObj.GetInterface<IGameManager>();

            LevelIndexText.text =
                "#" + (CachedLevelsData.Instance.LevelsData.LevelSequence.IndexOf(_gameManager.LevelID) + 1);

            _gameManager.StateChanged += (sender, args) => RefreshUIVisibility(true);
            RefreshUIVisibility(false);

            OptionsButton.onClick.AddListener(LaunchOptionsScreen);
        }

        private void LaunchOptionsScreen()
        {
            if (_uiToggler.IsContentVisible)
            {
                ScreenManager.OpenPanel(OptionsScreen.gameObject);
            }
        }

        private void RefreshUIVisibility(bool animate)
        {
            switch (_gameManager.State)
            {
                case GameManagerState.Idle:
                    _uiToggler.Show(animate);
                    break;
                case GameManagerState.Playing:
                    _uiToggler.Hide(animate);
                    break;
                case GameManagerState.GameOver:
                    _uiToggler.Show(animate);
                    break;
                default:
                    throw new ArgumentOutOfRangeException();
            }
        }
    }
}
