﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Assets.Scripts.Common;
using Assets.Scripts.Data;
using DG.Tweening;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

namespace Assets.Scripts.Game
{
    class GameOverPanel : MonoBehaviour
    {
        public Color WinColor;
        public Color LoseColor;
        public GameObject Panel;
        public Button RetryButton;
        public Button ShareButton;
        public Button NextLevelButton;
        public Text StatusText;
        public float EaseDuration = 0.2f;
        public Ease Ease = Ease.InQuad;

        public void Awake()
        {
            RetryButton.onClick.AddListener(() =>
            {
                //                Click.Instance.Play();
                SceneManager.LoadScene(SceneManager.GetActiveScene().name);
            });
            NextLevelButton.onClick.AddListener(() =>
            {
                Guid nextLevelID = ActiveLevelManager.Instance.GetActiveLevel();
                if (nextLevelID != Guid.Empty)
                {
                    LevelTransitionManager.Instance.TransitionToLevel(nextLevelID);
                }
                else
                {
                    MessageBoxLauncher.Instance.Show("No more levels!", MessageBoxButtons.OK, result =>
                    {
                        //                        Click.Instance.Play();
                        SceneManager.LoadScene("LevelSelect");
                    });
                }
            });
            ShareButton.onClick.AddListener(() =>
            {

            });

            GameObject gameManagerGameObj = GameObject.FindGameObjectWithTag(GlobalConstants.GameManagerTag);
            var gameManager = gameManagerGameObj.GetInterface<IGameManager>();
            gameManager.OnGameOver += GameManagerOnOnGameOver;
        }

        private void GameManagerOnOnGameOver(object sender, GameOverEventArgs gameOverEventArgs)
        {
            Show(gameOverEventArgs.IsLevelPassed);
        }

        private void Show(bool isLevelPassed)
        {
            if (isLevelPassed)
            {
                RetryButton.gameObject.SetActive(false);
                if (Helper.IsOnAndroid() || Helper.IsOnIOS())
                {
                    ShareButton.gameObject.SetActive(true);
                }
                else
                {
                    ShareButton.gameObject.SetActive(false);
                }
                NextLevelButton.gameObject.SetActive(true);
                StatusText.text = "PERFECT!";
                Panel.GetComponent<Image>().color = WinColor;
            }
            else
            {
                RetryButton.gameObject.SetActive(true);
                ShareButton.gameObject.SetActive(false);
                NextLevelButton.gameObject.SetActive(false);
                StatusText.text = "TRY AGAIN!";
                Panel.GetComponent<Image>().color = LoseColor;
            }

            Panel.gameObject.SetActive(true);

            RectTransform rectTransform = Panel.GetComponent<RectTransform>();
            float finalHeight = rectTransform.sizeDelta.y;
            rectTransform.sizeDelta = new Vector2(rectTransform.sizeDelta.x, 0);

            DOTween.To(() => rectTransform.sizeDelta.y,
                value => rectTransform.sizeDelta = new Vector2(rectTransform.sizeDelta.x, value), finalHeight,
                EaseDuration).SetEase(Ease);
        }
    }
}
