﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Assets.Scripts.Game
{
    class LevelTransitionManager : MonoBehaviour
    {
        public LevelTransitioner LevelTransitioner;

        public static LevelTransitionManager Instance
        {
            get
            {
                if (_instance == null)
                {
                    _instance = FindObjectOfType<LevelTransitionManager>();

                    //Tell unity not to destroy this object when loading a new scene!
                    DontDestroyOnLoad(_instance.gameObject);
                }

                return _instance;
            }
        }

        private static LevelTransitionManager _instance;

        public void Awake()
        {
            if (Instance == null)
            {
                //If I am the first instance, make me the Singleton
                _instance = this;
                DontDestroyOnLoad(this);
            }
            else
            {
                //If a Singleton already exists and you find
                //another reference in scene, destroy it!
                if (this != _instance)
                    Destroy(gameObject);
            }
        }

        public void LaunchLevel(Guid nextLevelID)
        {
            LevelTransitioner.LaunchLevel(nextLevelID);
        }

        public void TransitionToLevel(Guid nextLevelID)
        {
            LevelTransitioner.TransitionToLevel(nextLevelID);
        }
    }
}
