﻿using System;
using Assets.Scripts.Common;
using Assets.Scripts.Data;
using DG.Tweening;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

namespace Assets.Scripts.Game
{
    class LevelTransitioner : MonoBehaviour
    {
        public CanvasGroup CanvasGroup;
        public Text LevelText;
        public CanvasGroup TextCanvasGroup;
        public float TransitionDuration = 3;

        public void OnEnable()
        {
            BackButtonReceiver.SetActiveReceiver(gameObject);
        }

        public void OnDisable()
        {
            BackButtonReceiver.ReleaseReceiver(gameObject);
        }

        public void LaunchLevel(Guid nextLevelID)
        {
            gameObject.SetActive(true);
            CanvasGroup.alpha = 1;

            TextCanvasGroup.alpha = 0;

            LevelText.text = "#" + (CachedLevelsData.Instance.LevelsData.LevelSequence.IndexOf(nextLevelID) + 1);

            Sequence sequence = DOTween.Sequence();
            sequence.Append(DOTween.To(() => TextCanvasGroup.alpha,
                value => TextCanvasGroup.alpha = value, 1, TransitionDuration * 0.15f)
                .SetEase(Ease.InOutQuad).OnStart(() => SceneManager.LoadScene(nextLevelID.ToString())));
            sequence.Insert(TransitionDuration * 0.7f, DOTween.To(() => TextCanvasGroup.alpha,
                value => TextCanvasGroup.alpha = value, 0, TransitionDuration * 0.15f)
                .SetEase(Ease.InOutQuad));
            sequence.Insert(TransitionDuration * 0.85f, DOTween.To(() => CanvasGroup.alpha,
                value => CanvasGroup.alpha = value, 0, TransitionDuration * 0.15f)
                .SetEase(Ease.InOutQuad));

            sequence.OnComplete(() => gameObject.SetActive(false));
        }

        public void TransitionToLevel(Guid nextLevelID)
        {
            gameObject.SetActive(true);
            CanvasGroup.alpha = 0;

            TextCanvasGroup.alpha = 0;

            LevelText.text = "#" + (CachedLevelsData.Instance.LevelsData.LevelSequence.IndexOf(nextLevelID) + 1);

            Sequence sequence = DOTween.Sequence();
            sequence.Append(DOTween.To(() => CanvasGroup.alpha,
                value => CanvasGroup.alpha = value, 1, TransitionDuration * 0.15f).SetEase(Ease.InOutQuad));
            sequence.Append(DOTween.To(() => TextCanvasGroup.alpha,
                value => TextCanvasGroup.alpha = value, 1, TransitionDuration * 0.15f)
                .SetEase(Ease.InOutQuad).OnStart(() => SceneManager.LoadScene(nextLevelID.ToString())));
            sequence.Insert(TransitionDuration * 0.7f, DOTween.To(() => TextCanvasGroup.alpha,
                value => TextCanvasGroup.alpha = value, 0, TransitionDuration * 0.15f)
                .SetEase(Ease.InOutQuad));
            sequence.Insert(TransitionDuration * 0.85f, DOTween.To(() => CanvasGroup.alpha,
                value => CanvasGroup.alpha = value, 0, TransitionDuration * 0.15f)
                .SetEase(Ease.InOutQuad));

            sequence.OnComplete(() => gameObject.SetActive(false));
        }
    }
}
