﻿using System;
using Assets.Scripts.Common;
using UnityEngine;

namespace Assets.Scripts.Game
{
    class PressDetector : MonoBehaviour
    {
        public event EventHandler OnPress;
        public event EventHandler OnRelease;

        private bool _inputActive;
        private int _activeFingerID;

        public void OnDisable()
        {
            _inputActive = false;
        }

        public void Update()
        {
            if (Helper.IsOnDesktopPlatform())
            {
                if (Input.GetMouseButtonDown(0) && !Helper.IsPointerOverUI())
                {
                    _inputActive = true;
                    if (OnPress != null)
                    {
                        OnPress(this, EventArgs.Empty);
                    }
                }
                else if (_inputActive && Input.GetMouseButtonUp(0))
                {
                    _inputActive = false;
                    if (OnRelease != null)
                    {
                        OnRelease(this, EventArgs.Empty);
                    }
                }
            }
            else
            {
                if (!_inputActive)
                {
                    for (int i = 0; i < Input.touchCount; i++)
                    {
                        Touch touch = Input.GetTouch(i);
                        if (touch.phase == TouchPhase.Began && !Helper.IsTouchOverUI(touch))
                        {
                            _inputActive = true;
                            _activeFingerID = touch.fingerId;
                            if (OnPress != null)
                            {
                                OnPress(this, EventArgs.Empty);
                            }
                            break;
                        }
                    }
                    if (!_inputActive)
                    {
                        return;
                    }
                }

                if (_inputActive)
                {
                    Touch touch = new Touch();
                    for (int i = 0; i < Input.touchCount; i++)
                    {
                        touch = Input.GetTouch(i);
                        if (touch.fingerId == _activeFingerID)
                        {
                            break;
                        }
                    }

                    if (Input.touchCount == 0 || touch.fingerId != _activeFingerID)
                    {
                        _inputActive = false;
                        if (OnRelease != null)
                        {
                            OnRelease(this, EventArgs.Empty);
                        }
                    }
                }
            }
        }

        public Vector2 GetPressPosition()
        {
            if (Helper.IsOnDesktopPlatform())
            {
                return Camera.main.ScreenToWorldPoint(Input.mousePosition);
            }

            Touch touch = new Touch();
            for (int i = 0; i < Input.touchCount; i++)
            {
                touch = Input.GetTouch(i);
                if (touch.fingerId == _activeFingerID)
                {
                    break;
                }
            }

            return Camera.main.ScreenToWorldPoint(touch.position);
        }
    }
}
