﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Assets.Scripts.Common;
using Assets.Scripts.Data;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

namespace Assets.Scripts.Game
{
    [RequireComponent(typeof(UIToggler))]
    class RetryButtonManager : MonoBehaviour
    {
        private IGameManager _gameManager;
        private UIToggler _uiToggler;
        private Button _retryButton;

        public void Awake()
        {
            _uiToggler = GetComponent<UIToggler>();
            _retryButton = GetComponentInChildren<Button>();
        }

        public void Start()
        {
            GameObject gameManagerGameObj = GameObject.FindGameObjectWithTag(GlobalConstants.GameManagerTag);
            _gameManager = gameManagerGameObj.GetInterface<IGameManager>();

            _gameManager.StateChanged += (sender, args) => RefreshUIVisibility(true);
            RefreshUIVisibility(false);

            _retryButton.onClick.AddListener(LaunchOptionsScreen);
        }

        private void LaunchOptionsScreen()
        {
            if (_uiToggler.IsContentVisible)
            {
                SceneManager.LoadScene(SceneManager.GetActiveScene().name);
            }
        }

        private void RefreshUIVisibility(bool animate)
        {
            switch (_gameManager.State)
            {
                case GameManagerState.Idle:
                    _uiToggler.Hide(animate);
                    break;
                case GameManagerState.Playing:
                    _uiToggler.Show(animate);
                    break;
                case GameManagerState.GameOver:
                    _uiToggler.Hide(animate);
                    break;
                default:
                    throw new ArgumentOutOfRangeException();
            }
        }
    }
}
