﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DG.Tweening;
using UnityEngine;

namespace Assets.Scripts.Game
{
    class UIToggler : MonoBehaviour
    {
        public CanvasGroup Content;
        public float TransitionDuration = 0.3f;

        public bool IsContentVisible { get; private set; }

        private Tween _alphaTween;

        public void Awake()
        {
            IsContentVisible = Content.gameObject.activeSelf;
        }

        public void Show(bool animate = true)
        {
            if (!IsContentVisible)
            {
                if (_alphaTween != null && _alphaTween.IsActive())
                {
                    _alphaTween.Kill();
                }
                Content.gameObject.SetActive(true);
                if (animate)
                {
                    _alphaTween = DOTween.To(
                        () => Content.alpha, value => Content.alpha = value, 1, TransitionDuration)
                        .SetEase(Ease.InOutQuad);
                }
                else
                {
                    Content.alpha = 1;
                }
                IsContentVisible = true;
            }
        }

        public void Hide(bool animate = true)
        {
            if (IsContentVisible)
            {
                if (_alphaTween != null && _alphaTween.IsActive())
                {
                    _alphaTween.Kill();
                }
                if (animate)
                {
                    _alphaTween = DOTween.To(
                        () => Content.alpha, value => Content.alpha = value, 0, TransitionDuration)
                        .SetEase(Ease.InOutQuad).OnComplete(() => Content.gameObject.SetActive(false));
                }
                else
                {
                    Content.alpha = 0;
                    Content.gameObject.SetActive(false);
                }
                IsContentVisible = false;
            }
        }
    }
}
