﻿using System;
using System.Collections.Generic;
using Assets.Scripts.Data;
using Assets.Scripts.Game;
using UnityEngine;
using UnityEngine.UI;

namespace Assets.Scripts.LevelsScreenUtils
{
    class LevelsButtonPanel : MonoBehaviour
    {
        public LevelButton LevelButtonPrefab;

        public List<LevelButton> LevelButtons { get; private set; }

        public event EventHandler<LevelButtonClickEventArgs> LevelButtonClicked;

        public void Awake()
        {
            foreach (Transform child in transform)
            {
                Destroy(child.gameObject);
            }

            var playerData = PlayerManager.Instance.GetPlayerDataFromDisk();

            Guid currentUnfinishedLevelID = Guid.Empty;
            foreach (var level in CachedLevelsData.Instance.LevelsData.LevelSequence)
            {
                if (!playerData.CompletedLevels.Contains(level))
                {
                    currentUnfinishedLevelID = level;
                    break;
                }
            }

            LevelButtons = new List<LevelButton>(CachedLevelsData.Instance.LevelsData.LevelSequence.Count);
            var levelsState = GameHelper.GetAllLevelsState();

            for (int i = 0; i < CachedLevelsData.Instance.LevelsData.LevelSequence.Count; i++)
            {
                LevelButton levelButton = Instantiate(LevelButtonPrefab);
                levelButton.transform.SetParent(transform);
                levelButton.transform.localScale = Vector3.one;
                levelButton.Initialize(i, CachedLevelsData.Instance.LevelsData.LevelSequence[i],
                    levelsState[CachedLevelsData.Instance.LevelsData.LevelSequence[i]]);
                LevelButtons.Add(levelButton);

                int iCopy = i;
                levelButton.GetComponent<Button>().onClick.AddListener(() =>
                {
                    if (LevelButtonClicked != null)
                    {
                        LevelButtonClicked(this, new LevelButtonClickEventArgs(
                            levelButton, CachedLevelsData.Instance.LevelsData.LevelSequence[iCopy]));
                    }
                });
            }
        }
    }
}
